// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum anaPlotAlignPD23 {Left, Right, DoNotPlot}
public enum anaPivotStylesPD23 {Floor, Wide, Fibonacci, Camarilla, Woodies}

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Pivot Points.
	/// </summary>
	[Description("Daily Session Pivots.")]
	public class anaPivotsDailyV23: Indicator
	{
		#region Variables
		private	SolidBrush[]		brushes					= { new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black)};
		private DateTime 			firstBarSessionBegin	= Cbi.Globals.MinDate;
		private DateTime 			firstBarSessionEnd 		= Cbi.Globals.MinDate;
		private DateTime			plotStart				= Cbi.Globals.MinDate;
		private DateTime			cacheSessionBeginTmp   	= Cbi.Globals.MinDate;
		private DateTime			cacheSessionEndTmp		= Cbi.Globals.MinDate;
		private DateTime			cacheSessionDate		= Cbi.Globals.MinDate;
		private DateTime			currentDate				= Cbi.Globals.MinDate;
		private DateTime			priorDate				= Cbi.Globals.MinDate;
		private	double				currentHigh				= double.MinValue;
		private	double				currentLow				= double.MaxValue;
		private	double				currentClose			= 0;
		private	double				volSum					= 0;
		private	double				volPriceSum				= 0;
		private Bars				dailyBars				= null;
		private SolidBrush			textBrush				= new SolidBrush(Color.Red);
		private Font				textFont				= new Font("Arial", 12);
		private string				errorData1				= "Daily Pivots can only be displayed on intraday charts.";
		private string				errorData2				= "No daily data found. Please reload daily data or use CalcFromIntradayData.";
		private string				errorData3				= "Loading of daily data is incomplete. Please reload daily data or use CalcFromIntradayData.";
		private	string				errorData4				= "Insufficient historical data to calculate pivots. Please increase chart look back period.";
        private float				errorTextWidth			= 0;
        private float				errorTextHeight			= 0;
		private bool				existsHistDailyData		= false;
		private bool				isDailyDataLoaded		= false;
		private bool				plotPivots				= true;
		private bool				periodOpen				= false;
		private bool				skipPublicHoliday		= false; 
		private bool				isPublicHoliday			= false; 
		private bool				noDailyData				= false;
		private ArrayList			newSessionBarIdxArr1	= new ArrayList();
		private Data.PivotRange		pivotRangeType1			= PivotRange.Daily;
		private HLCCalculationMode	priorDayHLC				= HLCCalculationMode.CalcFromIntradayData;
		private anaPlotAlignPD23	plotLabels				= anaPlotAlignPD23.Right;
		private anaPivotStylesPD23 	pivotFormula			= anaPivotStylesPD23.Floor;
		private double				range					= 0;
		private	double				pp						= 0;
		private	double				r1						= 0;
		private	double				r2						= 0;
		private	double				r3						= 0;
		private	double				r4						= 0;
		private	double				s1						= 0;
		private	double				s2						= 0;
		private	double				s3						= 0;
		private	double				s4						= 0;
		private	double				rmid					= 0;
		private	double				r12mid					= 0;
		private	double				r23mid					= 0;
		private	double				r34mid					= 0;
		private	double				smid					= 0;
		private	double				s12mid					= 0;
		private	double				s23mid					= 0;
		private	double				s34mid					= 0;
		private double				midline					= 0;
		private double				previousVWAP			= 0;
		private double				previousHigh			= 0;
		private double				previousLow				= 0;
		private double				previousClose			= 0;
		private DateTime			sessionDateTmp			= Cbi.Globals.MinDate;
		private DateTime			sessionDateDaily		= Cbi.Globals.MinDate;
		private	StringFormat		stringFormatCenter		= new StringFormat();
		private	StringFormat		stringFormatFar			= new StringFormat();
		private	StringFormat		stringFormatNear		= new StringFormat();
		private double				userDefinedClose		= 0;
		private double				userDefinedHigh			= 0;
		private double				userDefinedLow			= 0;
		private int					countDown				= 2;
		private int					numberOfSessions		= 1;
		private int					sessionCount			= 0;
		private int					width					= 30;
		private int 				labelPosition			= 10;
		private bool				daySession				= false;
		private bool				showMidpivots			= false;
		private DateTime			publicHoliday0			= new DateTime (2009,01,19);
		private DateTime			publicHoliday1			= new DateTime (2009,02,16);
		private DateTime			publicHoliday2			= new DateTime (2009,05,25);
		private DateTime			publicHoliday3			= new DateTime (2009,07,03);
		private DateTime			publicHoliday4			= new DateTime (2009,09,07);
		private DateTime			publicHoliday5			= new DateTime (2009,11,26);
		private DateTime			publicHoliday6			= new DateTime (2010,01,18);
		private DateTime			publicHoliday7			= new DateTime (2010,02,15);
		private DateTime			publicHoliday8			= new DateTime (2010,05,31);
		private DateTime			publicHoliday9			= new DateTime (2010,07,05);
		private DateTime			publicHoliday10			= new DateTime (2010,09,06);
		private DateTime			publicHoliday11			= new DateTime (2010,11,25);
		private DateTime[]			publicHoliday			= new DateTime [12];
		
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"PP "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R1 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S1 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R2 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S2 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R3 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S3 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R4 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S4 "));
			
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"RMid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"SMid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R12Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S12Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R23Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S23Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R34Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S34Mid "));
			
			Add(new Plot(new Pen(Color.Lime,3), PlotStyle.Line,"Y-High "));
			Add(new Plot(new Pen(Color.OrangeRed,3), PlotStyle.Line,"Y-Low "));
			Add(new Plot(new Pen(Color.Yellow,3), PlotStyle.Line,"Y-Close "));
			Add(new Plot(new Pen(Color.MediumSpringGreen,1), PlotStyle.Line,"Y-Mid "));

			Add(new Plot(new Pen(Color.HotPink,1), PlotStyle.Line,"Y-VWAP"));

			Plots[0].Pen.DashStyle = DashStyle.Solid;
			Plots[1].Pen.DashStyle = DashStyle.Solid;
			Plots[2].Pen.DashStyle = DashStyle.Solid;
			Plots[3].Pen.DashStyle = DashStyle.Solid;
			Plots[4].Pen.DashStyle = DashStyle.Solid;
			Plots[5].Pen.DashStyle = DashStyle.Solid;
			Plots[6].Pen.DashStyle = DashStyle.Solid;
			Plots[7].Pen.DashStyle = DashStyle.Solid;
			Plots[8].Pen.DashStyle = DashStyle.Solid;
			Plots[9].Pen.DashStyle = DashStyle.Solid;
			Plots[10].Pen.DashStyle = DashStyle.Solid;
			Plots[11].Pen.DashStyle = DashStyle.Solid;
			Plots[12].Pen.DashStyle = DashStyle.Solid;
			Plots[13].Pen.DashStyle = DashStyle.Solid;
			Plots[14].Pen.DashStyle = DashStyle.Solid;
			Plots[15].Pen.DashStyle = DashStyle.Solid;
			Plots[16].Pen.DashStyle = DashStyle.Solid;
			Plots[17].Pen.DashStyle = DashStyle.Dot;
			Plots[18].Pen.DashStyle = DashStyle.Dot;
			Plots[19].Pen.DashStyle = DashStyle.Dot;
			Plots[20].Pen.DashStyle = DashStyle.Solid;
			Plots[21].Pen.DashStyle = DashStyle.Solid;
			
			AutoScale						= false;
			Overlay							= true;
			stringFormatNear.Alignment 		= StringAlignment.Near;
			stringFormatCenter.Alignment 	= StringAlignment.Center;
			stringFormatFar.Alignment		= StringAlignment.Far;
		}

		/// <summary>
		/// </summary>
		protected override void OnStartUp()
		{
			publicHoliday[0] = publicHoliday0;
			publicHoliday[1] = publicHoliday1;
			publicHoliday[2] = publicHoliday2;
			publicHoliday[3] = publicHoliday3;
			publicHoliday[4] = publicHoliday4;
			publicHoliday[5] = publicHoliday5;
			publicHoliday[6] = publicHoliday6;
			publicHoliday[7] = publicHoliday7;
			publicHoliday[8] = publicHoliday8;
			publicHoliday[9] = publicHoliday9;
			publicHoliday[10] = publicHoliday10;
			publicHoliday[11] = publicHoliday11;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (Bars == null)
				return; 
			if (!Bars.BarsType.IsIntraday && Bars.Period.Id != PeriodType.Day)
				return;
			if (Bars.Period.Id == PeriodType.Day && Bars.Period.Value > 1)
				return;
			if (Bars.Period.Id == PeriodType.Day && pivotRangeType1 == PivotRange.Daily)
			{
				textBrush.Color = ChartControl.AxisColor;
				DrawTextFixed("errortag1", errorData1, TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
				return;
			}			

			if (CurrentBar == 0)
			{	
				Bars.Session.GetNextBeginEnd(Time[0], out firstBarSessionBegin, out firstBarSessionEnd);
				plotStart = firstBarSessionEnd;
				plotStart = plotStart.AddDays(1);
				if (plotStart.DayOfWeek == DayOfWeek.Saturday)
					plotStart = plotStart.AddDays(2);
			}
			
			if (!isDailyDataLoaded)
			{
				if (priorDayHLC == HLCCalculationMode.DailyBars && Bars.BarsType.IsIntraday) 
				{
					Enabled = false;
					System.Threading.ThreadPool.QueueUserWorkItem(new System.Threading.WaitCallback(GetBarsNow));
					return;
				}
				existsHistDailyData = false;
				isDailyDataLoaded	= true;
			}
			if (priorDayHLC == HLCCalculationMode.DailyBars && !existsHistDailyData)
			{
				textBrush.Color = ChartControl.AxisColor;
				DrawTextFixed("errortag2", errorData2, TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
				return;
			}	

			IBar dailyBar;
			if (existsHistDailyData) 
			{
				dailyBar = dailyBars.Get(dailyBars.GetBar(currentDate));
				if (dailyBar.Time.Date != currentDate)
				{
					dailyBar = dailyBars.Get(dailyBars.GetBar(priorDate));
					if (dailyBar.Time.Date != priorDate)
					{
						noDailyData = true;
					}	
				}
			} 
			else 
				dailyBar = null;

			double	high		= existsHistDailyData ? dailyBar.High : High[0];
			double	low			= existsHistDailyData ? dailyBar.Low : Low[0];
			double	close		= existsHistDailyData ? dailyBar.Close : Close[0];

			DateTime lastBarTimeStamp1 = GetLastBarSessionDate(Time[0], Bars, pivotRangeType1);
			if (lastBarTimeStamp1 != currentDate || priorDayHLC == HLCCalculationMode.UserDefinedValues)
			{
				countDown = countDown -1;
				sessionCount = 1;
				if (!existsHistDailyData && daySession && numberOfSessions > 1)
					periodOpen = false;
				else
					periodOpen = true;
				if (countDown <1 && existsHistDailyData)
				{	
					currentClose = close;
					currentHigh = Math.Max(currentHigh, high);
					currentLow = Math.Min(currentLow, low);
				}
				priorDate = currentDate;
				currentDate = lastBarTimeStamp1;
				skipPublicHoliday = false;
				for (int i =0; i<12; i++)
				{
					if (publicHoliday[i].Date == priorDate)
						skipPublicHoliday = true;
				}
				if ((countDown<0 && !skipPublicHoliday) || priorDayHLC == HLCCalculationMode.UserDefinedValues)
				{
					range = (currentHigh - currentLow);						
					if (PivotFormula == anaPivotStylesPD23.Fibonacci)
					{
						pp				= (currentHigh + currentLow + currentClose) / 3;
						s1				= pp - 0.382*range;
						r1				= pp + 0.382*range;
						s2				= pp - 0.614*range;
						r2				= pp + 0.614*range;
						s3				= pp - range;
						r3				= pp + range;
						s4				= pp - 1.382*range;
						r4				= pp + 1.382*range;
					}
					else if (PivotFormula == anaPivotStylesPD23.Camarilla)
					{
						pp				= currentClose;
						s1				= currentClose - 1.1/12*range;
						r1				= currentClose + 1.1/12*range;
						s2				= currentClose - 1.1/6*range;
						r2				= currentClose + 1.1/6*range;
						s3				= currentClose - 1.1/4*range;
						r3				= currentClose + 1.1/4*range;
						s4				= currentClose - 1.1/2*range;
						r4				= currentClose + 1.1/2*range;
					}
					else
					{
						if (PivotFormula == anaPivotStylesPD23.Floor || PivotFormula == anaPivotStylesPD23.Wide)
							pp				= (currentHigh + currentLow + currentClose) / 3;
						else if (PivotFormula == anaPivotStylesPD23.Woodies)
							pp				= (currentHigh + currentLow + 2*currentClose) / 4;
						
						s1				= 2 * pp - currentHigh;
						r1				= 2 * pp - currentLow;
						s2				= pp - range;
						r2				= pp + range;
						if (PivotFormula == anaPivotStylesPD23.Floor || PivotFormula == anaPivotStylesPD23.Woodies)
						{
							s3				= s1 - range;
							r3				= r1 + range;
							s4				= s3 - (currentHigh - pp);
							r4				= r3 + (pp - currentLow);
						}
						else if (PivotFormula == anaPivotStylesPD23.Wide)
						{
							s3				= s2 - range;
							r3				= r2 + range;
							s4				= s3 - range;
							r4				= r3 + range;
						}
						smid			= 0.5*(pp+s1);
						rmid			= 0.5*(pp+r1);
						s12mid			= 0.5*(s1+s2);
						r12mid			= 0.5*(r1+r2);
						s23mid			= 0.5*(s2+s3);
						r23mid			= 0.5*(r2+r3);
						s34mid			= 0.5*(s3+s4);
						r34mid			= 0.5*(r3+r4);
					}
					previousHigh		= currentHigh;
					previousLow			= currentLow;
					previousClose		= currentClose;
					midline				= (currentHigh + currentLow) / 2;
					if (volSum > double.Epsilon)
						previousVWAP	= volPriceSum/volSum;
					else
						previousVWAP	= 0;
					if (noDailyData)
						plotPivots = false;
					else 
						plotPivots = true;
					noDailyData = false;
				}	
				if (!existsHistDailyData) 
				{
					if (numberOfSessions == 1 || !daySession)
					{
						currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
						if (!skipPublicHoliday)
						{
							currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : high;
							currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : low;
							volSum			= Volume[0];
							volPriceSum		= 0.5*(Median[0]+Typical[0])*Volume[0];
						}
						else
						{
							currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : Math.Max(currentHigh, high);
							currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : Math.Min(currentLow, low);
							volSum			= volSum + Volume[0];
							volPriceSum		= volPriceSum + 0.5*(Median[0]+Typical[0])*Volume[0];
						}
					}
					else if (!skipPublicHoliday)
					{	
						volSum			= 0;
						volPriceSum		= 0;
					}
				}
				else 
				{	
					if (!skipPublicHoliday)
					{
						currentHigh 	= double.MinValue;
						currentLow 		= double.MaxValue;
						volSum			= Volume[0];
						volPriceSum		= 0.5*(Median[0]+Typical[0])*Volume[0];
					}
					else
					{
						volSum			= volSum + Volume[0];
						volPriceSum		= volPriceSum + 0.5*(Median[0]+Typical[0])*Volume[0];
					}
				}
			}
			else if (Bars.FirstBarOfSession)
			{
				sessionCount = sessionCount +1;
				numberOfSessions = Math.Min(3, Math.Max(sessionCount, numberOfSessions));
				if (existsHistDailyData) 
				{
					volSum			= volSum + Volume[0];
					volPriceSum		= volPriceSum + 0.5*(Median[0]+Typical[0])*Volume[0];
				}	
				else if (sessionCount == 2 && daySession && !skipPublicHoliday)
				{
					currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
					currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : high;
					currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : low;
					volSum			= Volume[0];
					volPriceSum		= 0.5*(Median[0]+Typical[0])*Volume[0];
					periodOpen		= true;
				}
				else if (sessionCount == 2 || (sessionCount >2 && !daySession))
				{
					currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
					currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : Math.Max(currentHigh, high);
					currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : Math.Min(currentLow, low);
					volSum			= volSum + Volume[0];
					volPriceSum		= volPriceSum + 0.5*(Median[0]+Typical[0])*Volume[0];
					periodOpen		= true;
				}
				else 
					periodOpen = false;
			}
			else if (periodOpen)
			{	
				if (!existsHistDailyData) 
				{
					currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
					currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : Math.Max(currentHigh, high);
					currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : Math.Min(currentLow, low);
				}
				volSum			= volSum + Volume[0];
				volPriceSum		= volPriceSum + 0.5*(Median[0]+Typical[0])*Volume[0];
			}
			
			if (plotPivots && currentDate != Cbi.Globals.MinDate)
			{
				if (PivotFormula != anaPivotStylesPD23.Camarilla)
					PP.Set(pp);
				R1.Set(r1);
				S1.Set(s1);
				R2.Set(r2);
				S2.Set(s2);
				R3.Set(r3);
				S3.Set(s3);
				R4.Set(r4);
				S4.Set(s4);

				if (showMidpivots && PivotFormula != anaPivotStylesPD23.Camarilla && PivotFormula != anaPivotStylesPD23.Fibonacci)
				{
					RMid.Set(rmid);
					SMid.Set(smid);
					R12Mid.Set(r12mid);
					S12Mid.Set(s12mid);
					R23Mid.Set(r23mid);
					S23Mid.Set(s23mid);
					R34Mid.Set(r34mid);
					S34Mid.Set(s34mid);
				}	
				PreviousHigh.Set(previousHigh);
				PreviousLow.Set(previousLow);
				PreviousClose.Set(previousClose);
				Midline.Set(midline);
				if (Instrument.MasterInstrument.InstrumentType != Cbi.InstrumentType.Index && 
				 	Instrument.MasterInstrument.InstrumentType != Cbi.InstrumentType.Currency && previousVWAP > double.Epsilon
					&& priorDayHLC != HLCCalculationMode.UserDefinedValues)
					PVWAP.Set(previousVWAP);
				else
					PVWAP.Reset();
			}
			else
			{	
				PP.Set(0);
				R1.Set(0);
				S1.Set(0);
				R2.Set(0);
				S2.Set(0);
				R3.Set(0);
				S3.Set(0);
				R4.Set(0);
				S4.Set(0);
				RMid.Set(0);
				SMid.Set(0);
				R12Mid.Set(0);
				S12Mid.Set(0);
				R23Mid.Set(0);
				S23Mid.Set(0);
				R34Mid.Set(0);
				S34Mid.Set(0);
				PreviousHigh.Set(0);
				PreviousLow.Set(0);
				PreviousClose.Set(0);
				Midline.Set(0);
				PVWAP.Set(0);
			}
		}

		#region Properties

		/// <summary>
		/// </summary>
		[Description("Approach for calculation the prior day HLC values.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Prior Day High/Low/Close")]
		public Data.HLCCalculationMode PriorDayHLC
		{
			get { return priorDayHLC; }
			set { priorDayHLC = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Calculate pivots from Day Session")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Day Session Pivots")]
		public bool DaySession 
		{
			get { return daySession; }
			set { daySession = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Option to show MidPivots")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Midpivots")]
		public bool ShowMidpivots 
		{
			get { return showMidpivots; }
			set { showMidpivots = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Option where to plot labels")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot Labels")]
		public anaPlotAlignPD23 PlotLabels
		{
			get { return plotLabels; }
			set { plotLabels = value; }
		}
			
		/// <summary>
		/// </summary>
		[Description("Pivot Formula Type")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Pivot Formula")]
		public anaPivotStylesPD23 PivotFormula
		{
			get { return pivotFormula; }
			set { pivotFormula = value; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PP
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R2
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S2
		{
			get { return Values[4]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R3
		{
			get { return Values[5]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S3
		{
			get { return Values[6]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R4
		{
			get { return Values[7]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S4
		{
			get { return Values[8]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries RMid
		{
			get { return Values[9]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries SMid
		{
			get { return Values[10]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R12Mid
		{
			get { return Values[11]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S12Mid
		{
			get { return Values[12]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R23Mid
		{
			get { return Values[13]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S23Mid
		{
			get { return Values[14]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R34Mid
		{
			get { return Values[15]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S34Mid
		{
			get { return Values[16]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousHigh
		{
			get { return Values[17]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousLow
		{
			get { return Values[18]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousClose
		{
			get { return Values[19]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries Midline
		{
			get { return Values[20]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PVWAP
		{
			get { return Values[21]; }
		}

		/// <summary>
		/// close value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session close value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedClose
		{
			get { return userDefinedClose; }
			set { userDefinedClose = value; }
		}

		/// <summary>
		/// high value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session high value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedHigh
		{
			get { return userDefinedHigh; }
			set { userDefinedHigh = value; }
		}

		/// <summary>
		/// low value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session low value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedLow
		{
			get { return userDefinedLow; }
			set { userDefinedLow = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Width of the pivot lines as # of bars.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Line Width")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Distance of label from line.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Label Position")]
		public int LabelPosition
		{
			get { return labelPosition; }
			set { labelPosition = Math.Max(1, value); }
		}
	
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
        [GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 01 /No Trade Date")]
		public DateTime PublicHoliday0
		{
			get { return publicHoliday0;}
			set { publicHoliday0 = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 02 /No Trade Date")]
		public DateTime PublicHoliday1
		{
			get { return publicHoliday1;}
			set { publicHoliday1 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 03 /No Trade Date")]
		public DateTime PublicHoliday2
		{
			get { return publicHoliday2;}
			set { publicHoliday2 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 04 /No Trade Date")]
		public DateTime PublicHoliday3
		{
			get { return publicHoliday3;}
			set { publicHoliday3 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 05 /No Trade Date")]
		public DateTime PublicHoliday4
		{
			get { return publicHoliday4;}
			set { publicHoliday4 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 06 /No Trade Date")]
		public DateTime PublicHoliday5
		{
			get { return publicHoliday5;}
			set { publicHoliday5 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 07 /No Trade Date")]
		public DateTime PublicHoliday6
		{
			get { return publicHoliday6;}
			set { publicHoliday6 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 08 /No Trade Date")]
		public DateTime PublicHoliday7
		{
			get { return publicHoliday7;}
			set { publicHoliday7 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 09 /No Trade Date")]
		public DateTime PublicHoliday8
		{
			get { return publicHoliday8;}
			set { publicHoliday8 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 10 /No Trade Date")]
		public DateTime PublicHoliday9
		{
			get { return publicHoliday9;}
			set { publicHoliday9 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 11 /No Trade Date")]
		public DateTime PublicHoliday10
		{
			get { return publicHoliday10;}
			set { publicHoliday10 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[GridCategory("Public Holidays")]
		[Gui.Design.DisplayNameAttribute("Holiday 12 /No Trade Date")]
		public DateTime PublicHoliday11
		{
			get { return publicHoliday11;}
			set { publicHoliday11 = value;}
		}	
		
		#endregion

		#region Miscellaneous

		/// <summary>
		/// Have the .GetBars() call in a seperate thread. Reason: NT internal pool locking would not work if called in main thread.
		/// </summary>
		/// <param name="state"></param>
		private void GetBarsNow(object state)
		{
			if (Disposed)
				return;

			dailyBars			= Data.Bars.GetBars(Bars.Instrument, new Period(PeriodType.Day, 1, Bars.Period.MarketDataType), Bars.From, Bars.To, (Session) Bars.Session.Clone(), Data.Bars.SplitAdjust, Data.Bars.DividendAdjust);
			existsHistDailyData	= (dailyBars.Count <= 1) ? false : true;
			isDailyDataLoaded	= true;
			Enabled				= true;

			Cbi.Globals.SynchronizeInvoke.AsyncInvoke(new System.Windows.Forms.MethodInvoker(InvalidateNow), null);
		}

		private DateTime GetLastBarSessionDate(DateTime time, Data.Bars bars, PivotRange pivotRange)
		{
			if (time > cacheSessionEndTmp)
			{
				isPublicHoliday = false;
				for (int i =0; i<12; i++)
				{
					if (publicHoliday[i].Date == sessionDateTmp)
						isPublicHoliday = true;
				}
				Bars.Session.GetNextBeginEnd(time, out cacheSessionBeginTmp, out cacheSessionEndTmp);
				if (Bars.BarsType.IsIntraday)
					sessionDateTmp = TimeZoneInfo.ConvertTime(cacheSessionEndTmp,TimeZoneInfo.Local, Bars.Session.TimeZoneInfo).Date;
				else
					sessionDateTmp = time.Date;
				
				if(cacheSessionDate != sessionDateTmp) 
					{
						cacheSessionDate = sessionDateTmp;
						if (!isPublicHoliday && (newSessionBarIdxArr1.Count == 0 
							|| (newSessionBarIdxArr1.Count > 0 && CurrentBar > (int) newSessionBarIdxArr1[newSessionBarIdxArr1.Count - 1])))
								newSessionBarIdxArr1.Add(CurrentBar);
					}
			}
			return sessionDateTmp;
		}
		

		internal void InvalidateNow()
		{
			if (Disposed || ChartControl == null)
				return;

			ChartControl.Invalidate(true);
		}

		/// <summary>
        /// Overload this method to handle the termination of an indicator. Use this method to dispose of any resources vs overloading the Dispose() method.
		/// </summary>
		protected override void OnTermination()
		{
			if (dailyBars != null)
				dailyBars.Dispose();

			textBrush.Dispose();
			foreach (SolidBrush solidBrush in brushes)
				solidBrush.Dispose();
			stringFormatCenter.Dispose();
			stringFormatFar.Dispose();	
			stringFormatNear.Dispose();
		}
	
		/// <summary>
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="bounds"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;

			// plot error if data not complete
			DateTime lastBarSessionBegin = Cbi.Globals.MinDate;
			DateTime lastBarSessionEnd = Cbi.Globals.MinDate;
			Bars.Session.GetNextBeginEnd(Bars.Get(Math.Min(Bars.Count - 1, this.LastBarIndexPainted)).Time, out lastBarSessionBegin, out lastBarSessionEnd);
			textBrush.Color = ChartControl.AxisColor;
			SizeF errorSize	= graphics.MeasureString(errorData4, textFont);
			errorTextWidth	= errorSize.Width + 5;
			errorTextHeight	= errorSize.Height + 5;
			if (Bars.Count > 0 && plotStart.Date >= lastBarSessionEnd.Date && priorDayHLC != HLCCalculationMode.UserDefinedValues)
				graphics.DrawString(errorData4, textFont, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);

			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(Bars.BarsData.ChartStyle.BarWidthUI);
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				int				firstBarIdxToPaint	= -1;
				int				lastX				= -1;
				int				lastY				= -1;
				int				firstX				= -1;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries		series				= (DataSeries) Values[seriesCount];

				for (int i = newSessionBarIdxArr1.Count - 1; i >= 0; i--)
				{
					int prevSessionBreakIdx = (int) newSessionBarIdxArr1[i];
					if (prevSessionBreakIdx <= this.LastBarIndexPainted)
					{
						firstBarIdxToPaint = prevSessionBreakIdx;
						break;
					}
				}

				using (GraphicsPath	path = new GraphicsPath()) 
				{
					if (brush.Color != plot.Pen.Color)	
						brush = new SolidBrush(plot.Pen.Color);

					for (int idx = this.LastBarIndexPainted; idx >= Math.Max(this.FirstBarIndexPainted, this.LastBarIndexPainted - Width); idx--)
					{
						if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
							continue;
						else if (!series.IsValidPlot(idx))
							continue;

						if (idx < firstBarIdxToPaint)
							break;

						double	val = series.Get(idx);
						int		x	= ChartControl.GetXByBarIdx(BarsArray[0], idx);
						int		y	= ChartControl.GetYByValue(this, val);

						if (lastX >= 0)
						{
							if (y != lastY) // Problem here is, that last bar of old day has date of new day
								y = lastY;
							path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
						lastX	= x;
						lastY	= y;
						if (idx == this.LastBarIndexPainted || idx == this.LastBarIndexPainted-1)
							firstX	= x;
					}

					graphics.SmoothingMode = SmoothingMode.AntiAlias;
					graphics.DrawPath(plot.Pen, path);
					graphics.SmoothingMode = oldSmoothingMode;
					if (PlotLabels == anaPlotAlignPD23.Right)
					graphics.DrawString(plot.Name, ChartControl.Font, brush, firstX + LabelPosition + 20, lastY - ChartControl.Font.GetHeight() / 2, stringFormatNear);
					if (PlotLabels == anaPlotAlignPD23.Left)
					graphics.DrawString(plot.Name, ChartControl.Font, brush, lastX - LabelPosition, lastY - ChartControl.Font.GetHeight() / 2, stringFormatFar);
				}
			}
		}

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
		{
			if (pivotRange == PivotRange.Weekly)
			{
				DateTime periodStart = time.AddDays((6 - (((int) time.DayOfWeek) + 1) % 7));
				return periodStart.Date.AddDays(System.Math.Ceiling(System.Math.Ceiling(time.Date.Subtract(periodStart.Date).TotalDays) / 7) * 7).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}
		#endregion
	}
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaPivotsDailyV23[] cacheanaPivotsDailyV23 = null;

        private static anaPivotsDailyV23 checkanaPivotsDailyV23 = new anaPivotsDailyV23();

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public anaPivotsDailyV23 anaPivotsDailyV23(bool daySession, int labelPosition, anaPivotStylesPD23 pivotFormula, anaPlotAlignPD23 plotLabels, Data.HLCCalculationMode priorDayHLC, DateTime publicHoliday0, DateTime publicHoliday1, DateTime publicHoliday10, DateTime publicHoliday11, DateTime publicHoliday2, DateTime publicHoliday3, DateTime publicHoliday4, DateTime publicHoliday5, DateTime publicHoliday6, DateTime publicHoliday7, DateTime publicHoliday8, DateTime publicHoliday9, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return anaPivotsDailyV23(Input, daySession, labelPosition, pivotFormula, plotLabels, priorDayHLC, publicHoliday0, publicHoliday1, publicHoliday10, publicHoliday11, publicHoliday2, publicHoliday3, publicHoliday4, publicHoliday5, publicHoliday6, publicHoliday7, publicHoliday8, publicHoliday9, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public anaPivotsDailyV23 anaPivotsDailyV23(Data.IDataSeries input, bool daySession, int labelPosition, anaPivotStylesPD23 pivotFormula, anaPlotAlignPD23 plotLabels, Data.HLCCalculationMode priorDayHLC, DateTime publicHoliday0, DateTime publicHoliday1, DateTime publicHoliday10, DateTime publicHoliday11, DateTime publicHoliday2, DateTime publicHoliday3, DateTime publicHoliday4, DateTime publicHoliday5, DateTime publicHoliday6, DateTime publicHoliday7, DateTime publicHoliday8, DateTime publicHoliday9, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            if (cacheanaPivotsDailyV23 != null)
                for (int idx = 0; idx < cacheanaPivotsDailyV23.Length; idx++)
                    if (cacheanaPivotsDailyV23[idx].DaySession == daySession && cacheanaPivotsDailyV23[idx].LabelPosition == labelPosition && cacheanaPivotsDailyV23[idx].PivotFormula == pivotFormula && cacheanaPivotsDailyV23[idx].PlotLabels == plotLabels && cacheanaPivotsDailyV23[idx].PriorDayHLC == priorDayHLC && cacheanaPivotsDailyV23[idx].PublicHoliday0 == publicHoliday0 && cacheanaPivotsDailyV23[idx].PublicHoliday1 == publicHoliday1 && cacheanaPivotsDailyV23[idx].PublicHoliday10 == publicHoliday10 && cacheanaPivotsDailyV23[idx].PublicHoliday11 == publicHoliday11 && cacheanaPivotsDailyV23[idx].PublicHoliday2 == publicHoliday2 && cacheanaPivotsDailyV23[idx].PublicHoliday3 == publicHoliday3 && cacheanaPivotsDailyV23[idx].PublicHoliday4 == publicHoliday4 && cacheanaPivotsDailyV23[idx].PublicHoliday5 == publicHoliday5 && cacheanaPivotsDailyV23[idx].PublicHoliday6 == publicHoliday6 && cacheanaPivotsDailyV23[idx].PublicHoliday7 == publicHoliday7 && cacheanaPivotsDailyV23[idx].PublicHoliday8 == publicHoliday8 && cacheanaPivotsDailyV23[idx].PublicHoliday9 == publicHoliday9 && cacheanaPivotsDailyV23[idx].ShowMidpivots == showMidpivots && Math.Abs(cacheanaPivotsDailyV23[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV23[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV23[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && cacheanaPivotsDailyV23[idx].Width == width && cacheanaPivotsDailyV23[idx].EqualsInput(input))
                        return cacheanaPivotsDailyV23[idx];

            lock (checkanaPivotsDailyV23)
            {
                checkanaPivotsDailyV23.DaySession = daySession;
                daySession = checkanaPivotsDailyV23.DaySession;
                checkanaPivotsDailyV23.LabelPosition = labelPosition;
                labelPosition = checkanaPivotsDailyV23.LabelPosition;
                checkanaPivotsDailyV23.PivotFormula = pivotFormula;
                pivotFormula = checkanaPivotsDailyV23.PivotFormula;
                checkanaPivotsDailyV23.PlotLabels = plotLabels;
                plotLabels = checkanaPivotsDailyV23.PlotLabels;
                checkanaPivotsDailyV23.PriorDayHLC = priorDayHLC;
                priorDayHLC = checkanaPivotsDailyV23.PriorDayHLC;
                checkanaPivotsDailyV23.PublicHoliday0 = publicHoliday0;
                publicHoliday0 = checkanaPivotsDailyV23.PublicHoliday0;
                checkanaPivotsDailyV23.PublicHoliday1 = publicHoliday1;
                publicHoliday1 = checkanaPivotsDailyV23.PublicHoliday1;
                checkanaPivotsDailyV23.PublicHoliday10 = publicHoliday10;
                publicHoliday10 = checkanaPivotsDailyV23.PublicHoliday10;
                checkanaPivotsDailyV23.PublicHoliday11 = publicHoliday11;
                publicHoliday11 = checkanaPivotsDailyV23.PublicHoliday11;
                checkanaPivotsDailyV23.PublicHoliday2 = publicHoliday2;
                publicHoliday2 = checkanaPivotsDailyV23.PublicHoliday2;
                checkanaPivotsDailyV23.PublicHoliday3 = publicHoliday3;
                publicHoliday3 = checkanaPivotsDailyV23.PublicHoliday3;
                checkanaPivotsDailyV23.PublicHoliday4 = publicHoliday4;
                publicHoliday4 = checkanaPivotsDailyV23.PublicHoliday4;
                checkanaPivotsDailyV23.PublicHoliday5 = publicHoliday5;
                publicHoliday5 = checkanaPivotsDailyV23.PublicHoliday5;
                checkanaPivotsDailyV23.PublicHoliday6 = publicHoliday6;
                publicHoliday6 = checkanaPivotsDailyV23.PublicHoliday6;
                checkanaPivotsDailyV23.PublicHoliday7 = publicHoliday7;
                publicHoliday7 = checkanaPivotsDailyV23.PublicHoliday7;
                checkanaPivotsDailyV23.PublicHoliday8 = publicHoliday8;
                publicHoliday8 = checkanaPivotsDailyV23.PublicHoliday8;
                checkanaPivotsDailyV23.PublicHoliday9 = publicHoliday9;
                publicHoliday9 = checkanaPivotsDailyV23.PublicHoliday9;
                checkanaPivotsDailyV23.ShowMidpivots = showMidpivots;
                showMidpivots = checkanaPivotsDailyV23.ShowMidpivots;
                checkanaPivotsDailyV23.UserDefinedClose = userDefinedClose;
                userDefinedClose = checkanaPivotsDailyV23.UserDefinedClose;
                checkanaPivotsDailyV23.UserDefinedHigh = userDefinedHigh;
                userDefinedHigh = checkanaPivotsDailyV23.UserDefinedHigh;
                checkanaPivotsDailyV23.UserDefinedLow = userDefinedLow;
                userDefinedLow = checkanaPivotsDailyV23.UserDefinedLow;
                checkanaPivotsDailyV23.Width = width;
                width = checkanaPivotsDailyV23.Width;

                if (cacheanaPivotsDailyV23 != null)
                    for (int idx = 0; idx < cacheanaPivotsDailyV23.Length; idx++)
                        if (cacheanaPivotsDailyV23[idx].DaySession == daySession && cacheanaPivotsDailyV23[idx].LabelPosition == labelPosition && cacheanaPivotsDailyV23[idx].PivotFormula == pivotFormula && cacheanaPivotsDailyV23[idx].PlotLabels == plotLabels && cacheanaPivotsDailyV23[idx].PriorDayHLC == priorDayHLC && cacheanaPivotsDailyV23[idx].PublicHoliday0 == publicHoliday0 && cacheanaPivotsDailyV23[idx].PublicHoliday1 == publicHoliday1 && cacheanaPivotsDailyV23[idx].PublicHoliday10 == publicHoliday10 && cacheanaPivotsDailyV23[idx].PublicHoliday11 == publicHoliday11 && cacheanaPivotsDailyV23[idx].PublicHoliday2 == publicHoliday2 && cacheanaPivotsDailyV23[idx].PublicHoliday3 == publicHoliday3 && cacheanaPivotsDailyV23[idx].PublicHoliday4 == publicHoliday4 && cacheanaPivotsDailyV23[idx].PublicHoliday5 == publicHoliday5 && cacheanaPivotsDailyV23[idx].PublicHoliday6 == publicHoliday6 && cacheanaPivotsDailyV23[idx].PublicHoliday7 == publicHoliday7 && cacheanaPivotsDailyV23[idx].PublicHoliday8 == publicHoliday8 && cacheanaPivotsDailyV23[idx].PublicHoliday9 == publicHoliday9 && cacheanaPivotsDailyV23[idx].ShowMidpivots == showMidpivots && Math.Abs(cacheanaPivotsDailyV23[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV23[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV23[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && cacheanaPivotsDailyV23[idx].Width == width && cacheanaPivotsDailyV23[idx].EqualsInput(input))
                            return cacheanaPivotsDailyV23[idx];

                anaPivotsDailyV23 indicator = new anaPivotsDailyV23();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.DaySession = daySession;
                indicator.LabelPosition = labelPosition;
                indicator.PivotFormula = pivotFormula;
                indicator.PlotLabels = plotLabels;
                indicator.PriorDayHLC = priorDayHLC;
                indicator.PublicHoliday0 = publicHoliday0;
                indicator.PublicHoliday1 = publicHoliday1;
                indicator.PublicHoliday10 = publicHoliday10;
                indicator.PublicHoliday11 = publicHoliday11;
                indicator.PublicHoliday2 = publicHoliday2;
                indicator.PublicHoliday3 = publicHoliday3;
                indicator.PublicHoliday4 = publicHoliday4;
                indicator.PublicHoliday5 = publicHoliday5;
                indicator.PublicHoliday6 = publicHoliday6;
                indicator.PublicHoliday7 = publicHoliday7;
                indicator.PublicHoliday8 = publicHoliday8;
                indicator.PublicHoliday9 = publicHoliday9;
                indicator.ShowMidpivots = showMidpivots;
                indicator.UserDefinedClose = userDefinedClose;
                indicator.UserDefinedHigh = userDefinedHigh;
                indicator.UserDefinedLow = userDefinedLow;
                indicator.Width = width;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaPivotsDailyV23[] tmp = new anaPivotsDailyV23[cacheanaPivotsDailyV23 == null ? 1 : cacheanaPivotsDailyV23.Length + 1];
                if (cacheanaPivotsDailyV23 != null)
                    cacheanaPivotsDailyV23.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaPivotsDailyV23 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaPivotsDailyV23 anaPivotsDailyV23(bool daySession, int labelPosition, anaPivotStylesPD23 pivotFormula, anaPlotAlignPD23 plotLabels, Data.HLCCalculationMode priorDayHLC, DateTime publicHoliday0, DateTime publicHoliday1, DateTime publicHoliday10, DateTime publicHoliday11, DateTime publicHoliday2, DateTime publicHoliday3, DateTime publicHoliday4, DateTime publicHoliday5, DateTime publicHoliday6, DateTime publicHoliday7, DateTime publicHoliday8, DateTime publicHoliday9, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.anaPivotsDailyV23(Input, daySession, labelPosition, pivotFormula, plotLabels, priorDayHLC, publicHoliday0, publicHoliday1, publicHoliday10, publicHoliday11, publicHoliday2, publicHoliday3, publicHoliday4, publicHoliday5, publicHoliday6, publicHoliday7, publicHoliday8, publicHoliday9, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaPivotsDailyV23 anaPivotsDailyV23(Data.IDataSeries input, bool daySession, int labelPosition, anaPivotStylesPD23 pivotFormula, anaPlotAlignPD23 plotLabels, Data.HLCCalculationMode priorDayHLC, DateTime publicHoliday0, DateTime publicHoliday1, DateTime publicHoliday10, DateTime publicHoliday11, DateTime publicHoliday2, DateTime publicHoliday3, DateTime publicHoliday4, DateTime publicHoliday5, DateTime publicHoliday6, DateTime publicHoliday7, DateTime publicHoliday8, DateTime publicHoliday9, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.anaPivotsDailyV23(input, daySession, labelPosition, pivotFormula, plotLabels, priorDayHLC, publicHoliday0, publicHoliday1, publicHoliday10, publicHoliday11, publicHoliday2, publicHoliday3, publicHoliday4, publicHoliday5, publicHoliday6, publicHoliday7, publicHoliday8, publicHoliday9, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaPivotsDailyV23 anaPivotsDailyV23(bool daySession, int labelPosition, anaPivotStylesPD23 pivotFormula, anaPlotAlignPD23 plotLabels, Data.HLCCalculationMode priorDayHLC, DateTime publicHoliday0, DateTime publicHoliday1, DateTime publicHoliday10, DateTime publicHoliday11, DateTime publicHoliday2, DateTime publicHoliday3, DateTime publicHoliday4, DateTime publicHoliday5, DateTime publicHoliday6, DateTime publicHoliday7, DateTime publicHoliday8, DateTime publicHoliday9, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.anaPivotsDailyV23(Input, daySession, labelPosition, pivotFormula, plotLabels, priorDayHLC, publicHoliday0, publicHoliday1, publicHoliday10, publicHoliday11, publicHoliday2, publicHoliday3, publicHoliday4, publicHoliday5, publicHoliday6, publicHoliday7, publicHoliday8, publicHoliday9, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaPivotsDailyV23 anaPivotsDailyV23(Data.IDataSeries input, bool daySession, int labelPosition, anaPivotStylesPD23 pivotFormula, anaPlotAlignPD23 plotLabels, Data.HLCCalculationMode priorDayHLC, DateTime publicHoliday0, DateTime publicHoliday1, DateTime publicHoliday10, DateTime publicHoliday11, DateTime publicHoliday2, DateTime publicHoliday3, DateTime publicHoliday4, DateTime publicHoliday5, DateTime publicHoliday6, DateTime publicHoliday7, DateTime publicHoliday8, DateTime publicHoliday9, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaPivotsDailyV23(input, daySession, labelPosition, pivotFormula, plotLabels, priorDayHLC, publicHoliday0, publicHoliday1, publicHoliday10, publicHoliday11, publicHoliday2, publicHoliday3, publicHoliday4, publicHoliday5, publicHoliday6, publicHoliday7, publicHoliday8, publicHoliday9, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }
    }
}
#endregion
